// Library for nRF24L01+ Receiver.
// 2016-11-12  T. Nakagawa

#ifndef NRF24L01_H_
#define NRF24L01_H_
#define DEBUG

class NRF24L01 {
private:
  uint8_t ce_pin_;
  uint8_t cs_pin_;

  // Read a single register.
  uint8_t ReadRegister(uint8_t reg);

  // Read multiple registers.
  uint8_t ReadRegister(uint8_t reg, uint8_t *data, uint8_t len);

  // Write a single register.
  void WriteRegister(uint8_t reg, uint8_t val);

  // Write multiple registers.
  void WriteRegister(uint8_t reg, const uint8_t *data, uint8_t len);

public:
  enum CRC {
    CRC_NONE = 0x03,
    CRC_1BYTE = 0x0b,
    CRC_2BYTE = 0x0f,
  };

  enum DataRate {
    DR_1MBPS = 0x00,
    DR_2MBPS = 0x08,
    DR_250KBPS = 0x20,
  };

  // Initialize the module.
  // Mode: Any -> Standby
  NRF24L01(uint8_t ce_pin, uint8_t cs_pin);

  // Start receiving packets.
  // Mode: Standby -> RX
  void Enable();

  // Stop receiving packets.
  // Mode: RX -> Standby
  void Disable();

  // Check if RX FIFO is empty.
  // Mode: Any
  boolean Empty();

  // Disable CRC, or enable 1 or 2 bytes CRC.
  // Mode: Standby
  void SetCRC(CRC crc);

  // Set air data rate.
  // Mode: Standby
  void SetDataRate(DataRate dr);

  // Set channel (0~125).
  // Mode: Standby
  void SetChannel(uint8_t channel);

  // Set the number of RX pipes to be used (0~6).
  // Mode: Standby
  void SetPipes(uint8_t pipes);

  // Set address width.
  // Mode: Standby
  void SetAddressWidth(uint8_t aw);

  // Set RX address. The address length needs to be set by SetAddressWidth() beforehand.
  // Mode: Standby
  void SetAddress(uint8_t pipe, const uint8_t *adrs);

  // Set payload size.
  // Mode: Standby
  void SetPayloadSize(uint8_t size);

  // Read received data. The payload size needs to be set by SetPayloadSize() beforehand.
  // Mode: RX
  void Read(uint8_t *data);

  // Flush RX FIFO.
  // Mode: RX
  void Flush();

  // Read RPD status.
  // Mode: Standby/RX
  boolean TestRPD();

#ifdef DEBUG
  // Output debug information to Serial.
  // Mode: Standby
  void DumpRegisters();
#endif
};

#endif
